=== modified file 'testweave.py'
--- testweave.py	2005-06-30 08:40:59 +0000
+++ testweave.py	2005-07-07 07:57:12 +0000
@@ -533,7 +533,7 @@
 
 
 class Merge(TestBase):
-    """Versions that merge diverged parents"""
+    """Storage of versions that merge diverged parents"""
     def runTest(self):
         k = Weave()
 
@@ -567,6 +567,44 @@
         self.check_read_write(k)
 
 
+class Conflicts(TestBase):
+    """Test detection of conflicting regions during a merge.
+
+    A base version is inserted, then two descendents try to
+    insert different lines in the same place.  These should be
+    reported as a possible conflict and forwarded to the user."""
+    def runTest(self):
+        return  # NOT RUN
+        k = Weave()
+
+        k.add([], ['aaa', 'bbb'])
+        k.add([0], ['aaa', '111', 'bbb'])
+        k.add([1], ['aaa', '222', 'bbb'])
+
+        merged = k.merge([1, 2])
+
+        self.assertEquals([[['aaa']],
+                           [['111'], ['222']],
+                           [['bbb']]])
+
+
+
+class NonConflict(TestBase):
+    """Two descendants insert compatible changes.
+
+    No conflict should be reported."""
+    def runTest(self):
+        return  # NOT RUN
+        k = Weave()
+
+        k.add([], ['aaa', 'bbb'])
+        k.add([0], ['111', 'aaa', 'ccc', 'bbb'])
+        k.add([1], ['aaa', 'ccc', 'bbb', '222'])
+
+    
+    
+
+
 class AutoMerge(TestBase):
     def runTest(self):
         k = Weave()
@@ -582,7 +620,7 @@
 
         self.log('k._l=' + pformat(k._l))
 
-        m = list(k.merge_iter([0, 1, 2]))
+        m = list(k.mash_iter([0, 1, 2]))
 
         self.assertEqual(m,
                          ['header', 'aaa',

=== modified file 'weave.py'
--- weave.py	2005-07-01 04:07:49 +0000
+++ weave.py	2005-07-07 07:57:12 +0000
@@ -313,7 +313,7 @@
         # just by just counting as we go along.
 
         WFE = WeaveFormatError
-        
+
         for l in self._l:
             if isinstance(l, tuple):
                 isactive = None         # recalculate
@@ -341,6 +341,7 @@
                         if istack[-1] == v:
                             raise WFE("version %d deletes own text on line %d"
                                       % (v, lineno))
+                        # XXX
                         dset.add(v)
                 elif c == ']':
                     if v in dset:
@@ -382,7 +383,7 @@
         return list(self.get_iter(index))
 
 
-    def merge_iter(self, included):
+    def mash_iter(self, included):
         """Return composed version of multiple included versions."""
         included = frozenset(included)
         for origin, lineno, text in self._extract(included):
@@ -430,6 +431,23 @@
 
 
 
+    def merge(self, merge_versions):
+        """Automerge and mark conflicts between versions.
+
+        This returns a sequence, each entry describing alternatives
+        for a chunk of the file.  Each of the alternatives is given as
+        a list of lines.
+
+        If there is a chunk of the file where there's no diagreement,
+        only one alternative is given.
+        """
+
+        # approach: find the included versions common to all the
+        # merged versions
+        raise NotImplementedError()
+
+
+
     def _delta(self, included, lines):
         """Return changes from basis to new revision.
 

=== modified file 'weavefile.py'
--- weavefile.py	2005-07-01 03:39:34 +0000
+++ weavefile.py	2005-07-07 07:57:12 +0000
@@ -144,13 +144,13 @@
         if l == 'W\n':
             break
         elif l.startswith('. '):
-            w._l.append(l[2:])           # include newline
+            w._l.append(intern(l[2:]))  # include newline
         elif l.startswith(', '):
             w._l.append(l[2:-1])        # exclude newline
         else:
             assert l[0] in '{}[]', l
             assert l[1] == ' ', l
-            w._l.append((l[0], int(l[2:])))
+            w._l.append((intern(l[0]), int(l[2:])))
 
     return w
     

